#!/usr/bin/perl

use strict;

my ($artificialfile,$descriptionfile,@hitfile)=@ARGV;

my $full=1; # change this to 0 if you don't want to show coverage on 
            # the normal genomic variant of the gene is wished to be shown

open(F,$artificialfile);

$/=">";
my $seq="";

while(<F>) {
  chomp;

  if($_) {
    my $name;
    ($name)=/^(\S+)\s*/;
    my $pos=index($_,"\n");
    $seq=substr($_,$pos+1);
      
    die "ERROR 21: Wrong FASTA format: .$_." if (!$name || !$seq);
    
    $seq =~ tr/\n//d;
    $seq =~ tr/\r//d;

    $seq=lc($seq);
  }

}

close(F);

$/="\n";

my @mutation;

my %gcolor;  # to mark reads that hit correct genomic variant
my %mcolor; # to mark reads that hit mutated variant

my $m=1;
my $mlen=0;
open(F,$descriptionfile);
while(<F>) {
    chomp;
    my ($name,$type,$gstart,$gend,$change,$mstart,$mend,$comment)=split(/\t/);
    
    if($type ne "DUP") {
	for(my $i=$gstart;$i<=$gend;$i++) { push(@{$gcolor{$i}},$m);}  # normal variant might have overlapping locations for different mutations

	for(my $i=$mstart;$i<=$mend;$i++) { $mcolor{$i}=$m;} 

	push(@{$mutation[$m]},($name,$type,$gstart,$gend,$change,$mstart,$mend,$comment));

	if(($type ne "DUP")&&($type ne "DEL")) {
	    if($gend-$gstart+1>$mlen) { $mlen=$gend-$gstart+1;}
	    if($mend-$mstart+1>$mlen) { $mlen=$mend-$mstart+1;}
	}
    }

    $m++;
}
close(F);

#my @mcovg;
my @mexacthit;
my @maproxhit;
my @mminhit;
my @mmaxhit;
my @mstarts;
my @mends;

#my @gcovg;
my @gexacthit;
my @gaproxhit;
my @gminhit;
my @gmaxhit;

for(my $i=0;$i<=$#hitfile;$i++) {

    print STDERR "Process file ",$hitfile[$i],"\n";

    open(F,$hitfile[$i]);
    my $r=0;
    while(<F>) {
	chomp;
	$r++;
	my ($id1,$strand,$id2,$pos,$read,$qlty,$n,$chg)=split(/\t/);

	my @gn;
	my $mn=0;
	my @start;
	my @end;
	my $readlen=length($read);
	my %mutseen;
	my %mutidx;
	my $km=0;


	for(my $j=$pos+1;$j<=$pos+length($read);$j++) {
	    if($gcolor{$j}[0]) { # check if read hits the position of mutation on real gene

		for(my $k=0;$k<scalar(@{$gcolor{$j}});$k++) {
		    if(!$mutseen{$gcolor{$j}[$k]}) { $mutseen{$gcolor{$j}[$k]}=++$km;}
		    my $ki=$mutseen{$gcolor{$j}[$k]};
		    $gn[$ki]++;

		    if(!$start[$ki]) { 
			$mutidx{$ki}=$gcolor{$j}[$k]; 
			$start[$ki]=$j-$pos;
			$end[$ki]=$start[$ki];
		    }
		    else { $end[$ki]++;}

		}
	    }
	    if($mcolor{$j}) { # check if read hits the position of mutation on artificial gene

		$mn++;
		if(!$start[0]) { $start[0]=$j-$pos; $end[0]=$start[0];}
		else { $end[0]++;}
	    }
	}

	# the reads fully supporting the mutation will be the ones going across it

	if($mn>1) { # mutation was hit including more than one end
	  
	    my $m_i=$mcolor{$start[0]+$pos}; # idx of mutation hit

	    analyse_hit($seq,$_,$m_i,$pos,$start[0],$end[0],$strand,$chg,$readlen,$mutation[$m_i][1],1,$mn,$mutation[$m_i][2],$mutation[$m_i][3],$mutation[$m_i][5],$mutation[$m_i][6],\@mexacthit,\@maproxhit,\@mminhit,\@mmaxhit,\@mstarts,\@mends);

 	}
	else { # check if reads hits normal version of gene


	    for(my $k=1;$k<=$km;$k++) {


		if($gn[$k]>1) { # mutation was hit including more than one end
		
		    my $m_i=$mutidx{$k}; # idx of mutation hit
	    
		    analyse_hit($seq,$_,$m_i,$pos,$start[$k],$end[$k],$strand,$chg,$readlen,$mutation[$m_i][1],0,$gn[$k],$mutation[$m_i][5],$mutation[$m_i][6],$mutation[$m_i][2],$mutation[$m_i][3],\@gexacthit,\@gaproxhit,\@gminhit,\@gmaxhit);
		}
	    }
	}

    }
    close(F);

    print STDERR "Done file ",$hitfile[$i],"\n";

}

# now report the results


my $n=1;
# first print the definite mutations
print "Potential mutations:\n\n";
my $yes=0;
for(my $i=1;$i<$m;$i++) {

    #if($mcovg[$i]) {
    if($mstarts[$i] && $mstarts[$i]==$mutation[$i][5] && $mends[$i]==$mutation[$i][6]) { # reads go across mutation
	$mminhit[$i]-=$mlen;
	$mmaxhit[$i]+=$mlen;
	$yes=1; # found a mutation hit
	my $name=$mutation[$i][0];
	my $comment=$mutation[$i][7];
	print "***$n\n$name\n";
	print "Type: ";
	if($mutation[$i][1] eq "DEL") { print "deletion\n";}
	elsif($mutation[$i][1] eq "CHG") { print "substitution\n";}
	elsif($mutation[$i][1] eq "INS") { print "insertion\n";}
	print "Description:$comment\n";
	print "Supporting evidence:\n";
	print "normal:   ";

	my $gstart=$mutation[$i][2];
	my $gend=$mutation[$i][3];
	my $mstart=$mutation[$i][5];
	my $mend=$mutation[$i][6];

	my $lflank=$mstart-$mminhit[$i];
	if($gminhit[$i] && $gstart-$gminhit[$i]>$lflank) { $lflank=$gstart-$gminhit[$i];}
	print substr($seq,$gstart-$lflank,$lflank);
	my $insins="";
	if($mutation[$i][1] eq "INS") {
	    for(my $t=0;$t<length($mutation[$i][4]);$t++) { $insins.="-";print "-";}
	}
	else { print uc(substr($seq,$gstart,$gend-1-$gstart));}
	my $rflank=$mmaxhit[$i]-$mend+2;
	if($gmaxhit[$i] && $gmaxhit[$i]-$gend+2>$rflank) { $rflank=$gmaxhit[$i]-$gend+2;}
	print substr($seq,$gend-1,$rflank),"\n";

	print "mutation: ";
	print substr($seq,$mstart-$lflank,$lflank);
	my $insdel="";
	if($mutation[$i][1] eq "DEL") {
	    for(my $t=$gstart+1;$t<=$gend-1;$t++) { $insdel.="-";print "-";}
	}
	else { print uc($mutation[$i][4]);}
	print substr($seq,$mend-1,$rflank),"\n";

	print_read($i,\@mexacthit,"exact",$mutation[$i][1],"DEL",$insdel,$lflank,$rflank,$mstart,$mend);
	print_read($i,\@maproxhit,"approximate",$mutation[$i][1],"DEL",$insdel,$lflank,$rflank,$mstart,$mend);
	if($full) {
	    print "normal\ncoverage:\n";
	    my $n1=print_read($i,\@gexacthit,"exact",$mutation[$i][1],"INS",$insins,$lflank,$rflank,$gstart,$gend);
	    my $n2=print_read($i,\@gaproxhit,"approximate",$mutation[$i][1],"INS",$insins,$lflank,$rflank,$gstart,$gend);
	    #if(!($n1+$n2)) { print "\n";}
	}
	print "\n\n";
	$n++;
    }
}
if(!$yes) { print "NONE\n";}

sub print_read {
    my ($m,$hit,$hittype,$mutation,$instype,$insertion,$llen,$rlen,$start,$end)=@_;

    my $n=0;


    if($$hit[$m]) { $n=scalar(@{$$hit[$m]});}
    if($n) {
	if($n==1) { print "\tThere is ",$n," read with $hittype match:\n";}
	else { print "\tThere are ",$n," reads with $hittype match:\n";}
	for(my $j=0;$j<$n;$j++) {
	    my ($id1,$strand,$id2,$pos,$read,$qlty,$numb,$chg)=split(/\t/,$$hit[$m][$j]);
	    
	    my $lpoint=$llen-$start+$pos;
	    my $rpoint=$rlen-($pos+length($read)-$end+1);
	    
	    if($mutation eq $instype) {
		substr($read,$start-$pos,0)=$insertion;
	    }
	    print "          ";
	    for(my $t=0;$t<$lpoint;$t++) { print ".";}
	    print lc($read);
	    for(my $t=0;$t<$rpoint;$t++) { print ".";}
	    print "\n";
	}
    }

    return($n);
}


sub analyse_hit {
    my ($seq,$line,$m_i,$pos,$start,$end,$strand,$chg,$readlen,$mutation,$mutated,$mn,$gstart,$gend,$mstart,$mend,$exacthit,$aproxhit,$minhit,$maxhit,$mstarts,$mends)=@_;

    # first check if read could hit around as well (i.e. if hit supports mutation)
    my $readhit=substr($seq,$pos,$readlen);

    if(($mutated && ($mutation eq "DEL")) || (!$mutated && ($mutation eq "INS"))) { # this is a deletion mutation => deletion necessarily spans across mutation
	my $shift1=substr($seq,$gstart-$start,$readlen);
	my $shift2=substr($seq,$gend-$start-1,$readlen);
	    
	if($chg) { # see if there are any changes that are supported in between start+1 and end-1
	    my @c=split(/,/,$chg);
	    for(my $k=0;$k<=$#c;$k++) {
		my @e=split(/:/,$c[$k]);
		if($strand eq "+") {
		    substr($readhit,$e[0],1)="*";
		    substr($shift1,$e[0],1)="*";
		    substr($shift2,$e[0],1)="*";
		}
		else { # negative strand
		    substr($readhit,$readlen-$e[0]-1,1)="*";
		    substr($shift1,$readlen-$e[0]-1,1)="*";
		    substr($shift2,$readlen-$e[0]-1,1)="*";
		}
	    }
	}

	
	if(!($shift1 eq $readhit) && !($shift2 eq $readhit)) { # mutation was hit uniquely

	    #$$covg[$m_i]=1; # read spans across mutation => shows full covg

	    if($mutated) {
		if(!$$mstarts[$m_i] || $$mstarts[$m_i]>$start+$pos) { $$mstarts[$m_i]=$start+$pos;}
		if($$mends[$m_i]<$end+$pos) { $$mends[$m_i]=$end+$pos;}
	    }


	    if(!$chg) { # read doesn't show any modifications from initial mutated variant

		push(@{$$exacthit[$m_i]},$line); # perfect hit
		if(!$$minhit[$m_i]) { $$minhit[$m_i]=$pos; $$maxhit[$m_i]=$pos+$readlen-1;}
		else {
		    if($pos<$$minhit[$m_i]) { $$minhit[$m_i]=$pos;}
		    if($pos+$readlen-1>$$maxhit[$m_i]) { $$maxhit[$m_i]=$pos+$readlen-1;}
		}
	    }
	    else {
		push(@{$$aproxhit[$m_i]},$line);  # not exact hit
		if(!$$minhit[$m_i]) { $$minhit[$m_i]=$pos; $$maxhit[$m_i]=$pos+$readlen-1;}
		else {
		    if($pos<$$minhit[$m_i]) { $$minhit[$m_i]=$pos;}
		    if($pos+$readlen-1>$$maxhit[$m_i]) { $$maxhit[$m_i]=$pos+$readlen-1;}
		}
	    }
	}
    }
    elsif($mutation eq "CHG") { # this is a substitution mutation
		
	if($chg) { # see if there are any changes that are supported in between start+1 and end-1
	    my @c=split(/,/,$chg);
	    my $edge=0;

	    my $st=$start;
	    my $en=$end-2;
	    if($start+$pos>$mstart) { $st--;}
	    if($end+$pos<$mend) { $en++;}

	    for(my $k=0;$k<=$#c;$k++) {
		my @e=split(/:/,$c[$k]);

		if($strand eq "+") {
		    if($e[0]>=$st && $e[0]<=$en) { $mn--;}
		}
		else { # negative strand
		    if($readlen-$e[0]-1>=$st && $readlen-$e[0]-1<=$en) { $mn--;}
		}

	    }
		    
	    if($mstart-$pos==$start && $mend-$pos==$end) { $mn--;} # both ends are hit by the read
	}

	if($mn>1) { # read hit is still supporting some of the mutation
	    
	    #if($end-$start==$mend-$mstart) { $$covg[$m_i]=1;} # read spans across; sometimes the mutation might be too long to be spanned accross
		    
	    if($mutated) {
		if(!$$mstarts[$m_i] || $$mstarts[$m_i]>$start+$pos) { $$mstarts[$m_i]=$start+$pos;}
		if($$mends[$m_i]<$end+$pos) { $$mends[$m_i]=$end+$pos;}
	    }

	    if(!$chg) { # read doesn't show any modifications from initial mutated variant

		push(@{$$exacthit[$m_i]},$line); # perfect hit
		if(!$$minhit[$m_i]) { $$minhit[$m_i]=$pos; $$maxhit[$m_i]=$pos+$readlen-1;}
		else {
		    if($pos<$$minhit[$m_i]) { $$minhit[$m_i]=$pos;}
		    if($pos+$readlen-1>$$maxhit[$m_i]) { $$maxhit[$m_i]=$pos+$readlen-1;}
		}
	    }
	    else {
		push(@{$$aproxhit[$m_i]},$_);  # not exact hit
		if(!$$minhit[$m_i]) { $$minhit[$m_i]=$pos; $$maxhit[$m_i]=$pos+$readlen-1;}
		else {
		    if($pos<$$minhit[$m_i]) { $$minhit[$m_i]=$pos;}
		    if($pos+$readlen-1>$$maxhit[$m_i]) { $$maxhit[$m_i]=$pos+$readlen-1;}
		}
	    }
	}
    }
    elsif(($mutated && ($mutation eq "INS")) || (!$mutated &&($mutation eq "DEL"))) {

	my $shift1=substr($seq,$gstart-$start,$readlen);
	my $shift2=substr($seq,$gend-$end,$readlen);
		
	if($chg) { # see if there are any insertions that are supported in between start+1 and end-1
	    my @c=split(/,/,$chg);

	    my $st=$start;
	    my $en=$end-2;
	    if($start+$pos>$mstart) { $st--;}
	    if($end+$pos<$mend) { $en++;}

	    for(my $k=0;$k<=$#c;$k++) {
		my @e=split(/:/,$c[$k]);
		if($strand eq "+") {
		    if($e[0]>=$st && $e[0]<=$en) { $mn--;}
		    substr($readhit,$e[0],1)="*";
		    substr($shift1,$e[0],1)="*";
		    substr($shift2,$e[0],1)="*";
		}
		else { # negative strand
		    if($readlen-$e[0]-1>=$st && $readlen-$e[0]-1<=$en) { $mn--;}
		    substr($readhit,$readlen-$e[0]-1,1)="*";
		    substr($shift1,$readlen-$e[0]-1,1)="*";
		    substr($shift2,$readlen-$e[0]-1,1)="*";
		}

	    }

	    if($mstart-$pos==$start && $mend-$pos==$end) { $mn--;} # both ends are hit by the read
	}

	if($mn>1) { # read hit is still supporting some of the mutation (insertion)
		    
	    if(!($shift1 eq $readhit) && !($shift2 eq $readhit)) { # mutation was hit uniquely

		#if($end-$start==$mend-$mstart) { $$covg[$m_i]=1;} # read spans across

		if($mutated) {
		    if(!$$mstarts[$m_i] || $$mstarts[$m_i]>$start+$pos) { $$mstarts[$m_i]=$start+$pos;}
		    if($$mends[$m_i]<$end+$pos) { $$mends[$m_i]=$end+$pos;}
		}

		if(!$chg) { # read doesn't show any modifications from initial mutated variant

		    push(@{$$exacthit[$m_i]},$line); # perfect hit
		    if(!$$minhit[$m_i]) { $$minhit[$m_i]=$pos; $$maxhit[$m_i]=$pos+$readlen-1;}
		    else {
			if($pos<$$minhit[$m_i]) { $$minhit[$m_i]=$pos;}
			if($pos+$readlen-1>$$maxhit[$m_i]) { $$maxhit[$m_i]=$pos+$readlen-1;}
		    }
		}
		else {
		    push(@{$$aproxhit[$m_i]},$line);  # not exact hit
		    if(!$$minhit[$m_i]) { $$minhit[$m_i]=$pos; $$maxhit[$m_i]=$pos+$readlen-1;}
		    else {
			if($pos<$$minhit[$m_i]) { $$minhit[$m_i]=$pos;}
			if($pos+$readlen-1>$$maxhit[$m_i]) { $$maxhit[$m_i]=$pos+$readlen-1;}
		    }
		}
	    }
	}
    }
}
