#!/usr/bin/perl

use strict;

use File::Basename;
use Cwd;
use Cwd 'abs_path';

use FindBin;
use lib $FindBin::Bin;

my $usage = <<'&EOT&';
  Usage:
    get-brca-mutations.perl <read_files_directory> 
&EOT&

die $usage if (@ARGV<1);

my $dirfile= shift;

my $workdir=cwd();
my $scriptdir=$FindBin::Bin; # directory where all relevant files should be

# Run all human reads through Bowtie to align to BRCA1 and BRCA2 genes

print STDERR "Align reads to BRCA genes...\n";

opendir(IMD, $dirfile) || die("Cannot open directory $dirfile");
my @thefiles= readdir(IMD);
closedir(IMD);

$dirfile=abs_path($dirfile);

chdir "$scriptdir" or die printerr("ERROR: Can't cd to $scriptdir: $!\n");

unlink("$workdir/brca1.artificial.hits");
unlink("$workdir/brca2.artificial.hits");

foreach my $f (@thefiles) {
    unless ( ($f eq ".") || ($f eq "..") ) {

	my $nextfile=$dirfile."/".$f;

	print STDERR "Process $nextfile\n";

	my $zip=0;
	if(($nextfile =~ /\.gz$/) || ($nextfile =~ /\-gz$/) || ($nextfile =~ /\.z$/) || ($nextfile =~ /\-z$/) || ($nextfile =~ /\_z$/) || ($nextfile =~ /\.Z$/) ) {

	    system("gunzip -c $nextfile > /tmp/XXbrca1XX.fq");
	    $zip=1;
	}

	if($zip) {
	    system("bowtie -p 4 brca1.artificial /tmp/XXbrca1XX.fq >> $workdir/brca1.artificial.hits");
	    system("bowtie -p 4 brca2.artificial /tmp/XXbrca1XX.fq >> $workdir/brca2.artificial.hits");
	    system("rm /tmp/XXbrca1XX.fq");
	}
	else {
	    system("bowtie -p 4 brca1.artificial $nextfile >> $workdir/brca1.artificial.hits");
	    system("bowtie -p 4 brca2.artificial $nextfile >> $workdir/brca2.artificial.hits");
	}
	last;
    }
}

# now run the find mutations script and generate the report files
print STDERR "Look for mutations...\n";

system("$scriptdir/findmutations.perl brca1.artificial.fa brca1.mutations $workdir/brca1.artificial.hits > $workdir/brca1.report");
system("$scriptdir/findmutations.perl brca2.artificial.fa brca2.mutations $workdir/brca2.artificial.hits > $workdir/brca2.report");

# change to initial working directory 
chdir "$workdir" or die printerr("ERROR: Can't cd to $workdir: $!\n");

sub printerr {
    my $message=$_[0];
    print STDERR $message;
    return($message);
}
